package webhash

import (
	"html/template"
	"net/http"

	"appengine"
	"appengine/datastore"
	"appengine/user"
)

// datastore Entity type
type UserData struct {
	Value string
}

func init() {
	http.HandleFunc("/", rootHandler)
}

func rootHandler(w http.ResponseWriter, r *http.Request) {
	c := appengine.NewContext(r)
	u := user.Current(c)
	url, _ := user.LogoutURL(c, "/")

	// Handle user sign in.
	if u == nil {
		url, _ := user.LoginURL(c, "/")
		signInTemplate.Execute(w, url)
		return
	}

	// Retrieve or store a value
	k := datastore.NewKey(c, "UserData", u.String(), 0, nil)
	ud := UserData{
		Value: r.FormValue("value"),
	}
	if ud.Value == "" {
		datastore.Get(c, k, &ud)
		if ud.Value == "" {
			ud.Value = "null"
		}
	} else {
		datastore.Put(c, k, &ud)
	}

	// Generate the page
	rootTemplate.Execute(w, struct {
		Value      string
		SignoutURL string
	}{
		Value:      ud.Value,
		SignoutURL: url,
	})

}

// User sign in template
var signInTemplate = template.Must(template.New("signIn").Parse(signInTemplateHTML))

const signInTemplateHTML = `
<!DOCTYPE HTML>
<html>
	<head>
		<title>WebHash Go</title>
		<meta charset="utf-8">
	</head>
	<body>
        <div><a href="{{.}}">sign in or register</a></div>
	</body>
</html>
`

// Main page displaying user's value
var rootTemplate = template.Must(template.New("root").Parse(rootTemplateHTML))

const rootTemplateHTML = `
<!DOCTYPE HTML>
<html>
	<head>
		<title>WebHash Go</title>
		<meta charset="utf-8">
	</head>
	<body>
		{{.Value}}
        <div><a href="{{.SignoutURL}}">sign out</a></div>
	</body>
</html>
`
